/* hexedit -- Hexadecimal Editor for Binary Files
   Copyright (C) 1998 Pixel (Pascal Rigaux)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*/
#include "hexedit.h"


/*******************************************************************************/
/* Global variables */
/*******************************************************************************/
INT lastEditedLoc, biggestLoc, fileSize;
INT mark_min, mark_max, mark_set;
INT base, oldbase;
int normalSpaces, cursor, cursorOffset, hexOrAscii;
int cursor, blocSize, lineLength, colsUsed, page;
int isReadOnly, fd, nbBytes, oldcursor, oldattr, oldcursorOffset;
int sizeCopyBuffer, *bufferAttr;
char *progName, *fileName, *baseName;
unsigned char *buffer, *copyBuffer;
typePage *edited;

char *lastFindFile = NULL, *lastYankToAFile = NULL, *lastAskHexString = NULL, *lastAskAsciiString = NULL, *lastFillWithStringHexa = NULL, *lastFillWithStringAscii = NULL;


modeParams modes[LAST] = {
  { 8, 16, 256 },
  { 4, 0, 0 },
};
modeType mode = maximized;
int colored = FALSE;

char * usage = "usage: %s [-s | --sector] [-m | --maximize]"
#ifdef HAVE_COLORS 
     "\n"
     " [--color]\n"
     " [--color-null-fg {<color> | auto}]\n"
     " [--color-null-bg {<color> | auto}]\n"
     " [--color-control-fg {<color> | auto}]\n"
     " [--color-control-bg {<color> | auto}]\n"
     " [--color-normal-fg {<color> | auto}]\n"
     " [--color-normal-bg {<color> | auto}]\n"
     " [--color-extended-fg {<color> | auto}]\n"
     " [--color-extended-bg {<color> | auto}]\n"
     " [--bold]\n"
#endif 
     " [-h | --help] filename\n";

#ifdef HAVE_COLORS 
int color_null_fg     = COLOR_RED;
int color_control_fg  = COLOR_GREEN;
int color_extended_fg = COLOR_BLUE;
int color_normal_fg   = -1;
int color_null_bg     = -1;
int color_control_bg  = -1;
int color_extended_bg = -1;
int color_normal_bg   = -1;
int bold = FALSE;

int color_name_to_number(char *name)
{
  int i;
  static struct {
    char *name;
    int number;
  } name_to_number[] = {
    { "black",   COLOR_BLACK },
    { "red",     COLOR_RED },
    { "green",   COLOR_GREEN },
    { "yellow",  COLOR_YELLOW },
    { "blue",    COLOR_BLUE },
    { "magenta", COLOR_MAGENTA },
    { "cyan",    COLOR_CYAN },
    { "white",   COLOR_WHITE },
    { "auto",    -1 },
    { NULL, -1 }
  };

  for (i = 0; name_to_number[i].name; i++) {
    if (!strcmp(name, name_to_number[i].name))
      return name_to_number[i].number;
  }

  return -1;
}
#endif


/*******************************************************************************/
/* main */
/*******************************************************************************/
int main(int argc, char **argv)
{
  progName = basename(argv[0]);
  argv++; argc--;

  for (; argc > 0; argv++, argc--) 
    {
      if (streq(*argv, "-s") || streq(*argv, "--sector"))
	mode = bySector;
      else if (streq(*argv, "-m") || streq(*argv, "--maximize"))
	mode = maximized;
#ifdef HAVE_COLORS
      else if (streq(*argv, "--color"))
	colored = TRUE;
      else if (streq(*argv, "--color-null-fg") && (argc > 1)) {
	argv++; argc--;
	color_null_fg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--color-null-bg") && (argc > 1)) {
	argv++; argc--;
	color_null_bg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--color-control-fg") && (argc > 1)) {
	argv++; argc--;
	color_control_fg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--color-control-bg") && (argc > 1)) {
	argv++; argc--;
	color_control_bg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--color-extended-fg") && (argc > 1)) {
	argv++; argc--;
	color_extended_fg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--color-extended-bg") && (argc > 1)) {
	argv++; argc--;
	color_extended_bg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--color-normal-fg") && (argc > 1)) {
	argv++; argc--;
	color_normal_fg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--color-normal-bg") && (argc > 1)) {
	argv++; argc--;
	color_normal_bg = color_name_to_number(*argv);
      }
      else if (streq(*argv, "--bold"))
	bold = TRUE;
#endif
      else if (streq(*argv, "--")) {
	argv++; argc--;
	break;
      } else if (*argv[0] == '-')

	DIE(usage)
      else break;
    }
  if (argc > 1) DIE(usage);

  init();
  if (argc == 1) {
    fileName = strdup(*argv); 
    openFile();
  }
  initCurses();
  if (fileName == NULL) {
    if (!findFile()) {
      exitCurses();
      DIE("%s: No such file\n");
    }
    openFile();
  }
  readFile();
  do display();
  while (key_to_function(getch()));
  quit();
  return 0; /* for no warning */
}



/*******************************************************************************/
/* other functions */
/*******************************************************************************/
void init(void)
{
  page = 0; /* page == 0 means initCurses is not done */
  normalSpaces = 3;
  hexOrAscii = TRUE;
  copyBuffer = NULL;
  edited = NULL;
}

void quit(void)
{
  exitCurses();
  free(fileName);
  free(buffer);
  free(bufferAttr);
  FREE(copyBuffer);
  discardEdited();
  FREE(lastFindFile); FREE(lastYankToAFile); FREE(lastAskHexString); FREE(lastAskAsciiString); FREE(lastFillWithStringHexa); FREE(lastFillWithStringAscii);
  exit(0);
}





